@@ -26,7 +26,10 @@ smooth.construct.mpi.smooth.spec<- function(object, data, knots)
        for (i in (q+2):(q+m+2)) {xk[i]<-xk[q+1]+(i-q-1)*(xk[m+3]-xk[m+2])}
      }
   if (length(xk)!=nk) # right number of knots?
-  stop(paste("there should be ", nk," supplied knots"))
+   stop(paste("there should be ", nk," supplied knots"))
+  if (!is.null(object$point.con[[1]])) ## a point constraint is supplied?
+    stop(paste("'mpi' smooth does not work with a point constraint; use 'miso' for a start-at-zero constraint, or 'mifo' for a finish-at-zero constraint"))
+
   ##################################################
   ## setting column centering constraint... 
   #  get model matrix-------------
@@ -72,12 +75,13 @@ smooth.construct.mpi.smooth.spec<- function(object, data, knots)
     object$S[[1]] <- crossprod(P)
   }
   object$p.ident <- rep(TRUE,q-1) ## p.ident is an indicator of which coefficients must be positive (exponentiated)
+
   object$rank <- ncol(object$X)-1  # penalty rank
   object$null.space.dim <- m+1  # dim. of unpenalized space
   object$C <- matrix(0, 0, ncol(X)) # to have no other constraints 
   object$knots <- xk;
   object$m <- m;
-  object$df<-ncol(object$X)     # maximum DoF (if unconstrained)
+  object$df<-ncol(object$X)     # maximum DoF 
 
   ## get model matrix for 1st and 2nd derivatives of the smooth...
   h <- (max(x)-min(x))/(q-m-1) ## distance between two adjacent knots
@@ -91,28 +95,26 @@ smooth.construct.mpi.smooth.spec<- function(object, data, knots)
 
 ## Prediction matrix for the `mpi` smooth class... 
 
-
 Predict.matrix.mpi.smooth<-function(object,data)
 ## prediction method function for the `mpi' smooth class
-{ m <- object$m+1; # spline order, m+1=3 default for cubic spline
+{ m <- object$m # spline order, m+1=3 default for cubic spline
   q <- object$df +1
  # Sig <- matrix(as.numeric(rep(1:q,q)>=rep(1:q,each=q)),q,q) ## coef summation matrix
   Sig <- matrix(1,q,q)  ## coef summation matrix
   Sig[upper.tri(Sig)] <-0
   ## find spline basis inner knot range...
-  ll <- object$knots[m+1];ul <- object$knots[length(object$knots)-m]
-  m <- m + 1
+  ll <- object$knots[m+2];ul <- object$knots[length(object$knots)-m-1]
   x <- data[[object$term]]
   n <- length(x)
   ind <- x<=ul & x>=ll ## data in range
   if (sum(ind)==n) { ## all in range
-     X <- spline.des(object$knots,x,m)$design
+     X <- spline.des(object$knots,x,m+2)$design
      X <- X%*%Sig 
   } else { ## some extrapolation needed 
      ## matrix mapping coefs to value and slope at end points...
-     D <- spline.des(object$knots,c(ll,ll,ul,ul),m,c(0,1,0,1))$design
+     D <- spline.des(object$knots,c(ll,ll,ul,ul),m+2,c(0,1,0,1))$design
      X <- matrix(0,n,ncol(D)) ## full predict matrix
-     if (sum(ind)> 0)  X[ind,] <- spline.des(object$knots,x[ind],m)$design ## interior rows
+     if (sum(ind)> 0)  X[ind,] <- spline.des(object$knots,x[ind],m+2)$design ## interior rows
      ## Now add rows for linear extrapolation...
      ind <- x < ll 
      if (sum(ind)>0) X[ind,] <- cbind(1,x[ind]-ll)%*%D[1:2,]
@@ -123,6 +125,219 @@ Predict.matrix.mpi.smooth<-function(object,data)
   X
 }
 
+
+##############################################################################################
+### Adding Monotone increasing SCOP-spline construction with a 'start at zero' constraint,
+### a SCOP-spline additionally constrained to be zero at a start, at the left-end point of the covariate range...
+#############################################################################################
+
+smooth.construct.miso.smooth.spec<- function(object, data, knots)
+## construction of the monotone increasing smooth with a 'start-at-zero' constraint;
+## achieved simply by setting the first (m+1) spline coefficients to zero...
+{ 
+  m <- object$p.order[1]
+  if (is.na(m)) m <- 2 ## default for cubic spline
+  if (m<1) stop("silly m supplied")
+  if (object$bs.dim<0) object$bs.dim <- 10 ## default
+  q <- object$bs.dim 
+  nk <- q+m+2 ## number of knots
+  if (nk<=0) stop("k too small for m")
+  x <- data[[object$term]]  ## the data
+  xk <- knots[[object$term]] ## will be NULL if none supplied
+  if (is.null(xk)) { # space knots evenly through data
+       n<-length(x)
+       xk<-rep(0,q+m+2)
+       xk[(m+2):(q+1)]<-seq(min(x),max(x),length=q-m)
+       for (i in 1:(m+1)) {xk[i]<-xk[m+2]-(m+2-i)*(xk[m+3]-xk[m+2])}
+       for (i in (q+2):(q+m+2)) {xk[i]<-xk[q+1]+(i-q-1)*(xk[m+3]-xk[m+2])}
+     }
+  ## move m knots, xk[m+3],...,xk[m+3+m-1] (2nd,...,(m+1) inner knots), to the first inner knot xk[m+2],
+  ## join these m knots with the constrained xk[m+2] knot to avoid a plateau start...
+  xk[(m+3):(m+2+m)] <- xk[m+2]
+  if (length(xk)!=nk) # right number of knots?
+   stop(paste("there should be ", nk," supplied knots"))
+  if (!is.null(object$point.con[[1]])) ## a point constraint is supplied?
+    stop(paste("'miso' smooth works only with a 'start at zero' constraint; 'pc' argument does not work here"))
+  ## get model matrix...
+  X1 <- splineDesign(xk,x,ord=m+2)
+  ## get unconstrained matrix Sigma and remove the first m+1 columns and rows...
+  Sig <- matrix(1,q,q)  ## coef summation matrix
+  Sig[upper.tri(Sig)] <-0
+  ind <- 1:(m+1) ## 1:3;
+  ## Sig[,ind] <- 0; Sig[ind,] <- 0
+  Sig <- Sig[-ind,-ind]
+  X <- X1[,-ind]%*%Sig # drop (m+1) start terms, model submatrix for the scop-term
+  object$X <- X # the final model matrix
+  object$P <- list()
+  object$S <- list()
+  object$Sigma <- Sig
+
+  if (!object$fixed) # create the penalty matrix
+  { P <- diff(diag(q-length(ind)),difference=1)
+    object$P[[1]] <- P
+    object$S[[1]] <- crossprod(P)
+  }
+  object$p.ident <- rep(TRUE,q-length(ind)) ## p.ident is an indicator of which coefficients must be positive (exponentiated)
+  object$n.zero <- ind
+  object$rank <- ncol(object$X)-1  # penalty rank
+  object$null.space.dim <- m+1  # dim. of unpenalized space
+  object$C <- matrix(0, 0, ncol(X)) # to have no other constraints 
+  object$knots <- xk;
+  object$m <- m;
+  object$df<-ncol(object$X)     # maximum DoF 
+
+  ## get model matrix for 1st and 2nd derivatives of the smooth...
+  h <- (max(x)-min(x))/(q-m-1) ## distance between two adjacent knots
+  object$Xdf1 <- splineDesign(xk,x,ord=m+1)[,2:(q-1)]/h ## ord is by one less for the 1st derivative
+  object$Xdf2 <- splineDesign(xk,x,ord=m)[,2:(q-2)]/h^2 ## ord is by two less for the 2nd derivative
+
+  class(object)<-"miso.smooth"  # Give object a class
+  object
+}
+
+
+## Prediction matrix for the `miso` smooth class... 
+Predict.matrix.miso.smooth<-function(object,data)
+## prediction method function for the `miso' smooth class
+{ m <- object$m # spline order, m+1=3 default for cubic spline
+  q <- object$bs.dim ## object$df +1 ## ?????
+ # Sig <- matrix(as.numeric(rep(1:q,q)>=rep(1:q,each=q)),q,q) ## coef summation matrix
+  Sig <- matrix(1,q,q)  ## coef summation matrix
+  Sig[upper.tri(Sig)] <-0
+  ind <- object$n.zero ## 1:(m+1)
+  Sig[,ind] <- 0; Sig[ind,] <- 0
+  ## find spline basis inner knot range...
+  ll <- object$knots[m+2];ul <- object$knots[length(object$knots)-m-1]
+  x <- data[[object$term]]
+  n <- length(x)
+  ind <- x<=ul & x>=ll ## data in range
+  if (sum(ind)==n) { ## all in range
+     X <- spline.des(object$knots,x,m+2)$design
+     X <- X%*%Sig 
+  } else { ## some extrapolation needed 
+     ## matrix mapping coefs to value and slope at end points...
+     D <- spline.des(object$knots,c(ll,ll,ul,ul),m+2,c(0,1,0,1))$design
+     X <- matrix(0,n,ncol(D)) ## full predict matrix
+     if (sum(ind)> 0)  X[ind,] <- spline.des(object$knots,x[ind],m+2)$design ## interior rows
+     ## Now add rows for linear extrapolation...
+     ind <- x < ll 
+     if (sum(ind)>0) X[ind,] <- cbind(1,x[ind]-ll)%*%D[1:2,]
+     ind <- x > ul
+     if (sum(ind)>0) X[ind,] <- cbind(1,x[ind]-ul)%*%D[3:4,]
+     X <- X%*%Sig 
+  }
+  X
+}
+
+
+
+
+##############################################################################################
+### Adding Monotone increasing SCOP-spline construction with an 'finish at zero' constraint,
+### a SCOP-spline additionally constrained to be zero at the end, at the right-end point of 
+### the covariate range...
+#############################################################################################
+
+
+smooth.construct.mifo.smooth.spec<- function(object, data, knots)
+## construction of the monotone increasing smooth with a 'finish-at-zero' constraint;
+## achieved simply by setting the last (m+1) spline coefficients to zero...
+{ 
+  if (!is.null(object$point.con[[1]])) ## a point constraint is supplied?
+    stop(paste("'mifo' smooth works only with a 'finish at zero' constraint; 'pc' argument does not work here"))
+  m <- object$p.order[1]
+  if (is.na(m)) m <- 2 ## default for cubic spline
+  if (m<1) stop("silly m supplied")
+  if (object$bs.dim<0) object$bs.dim <- 10 ## default
+  q <- object$bs.dim 
+  nk <- q+m+2 ## number of knots
+  if (nk<=0) stop("k too small for m")
+  x <- data[[object$term]]  ## the data
+  xk <- knots[[object$term]] ## will be NULL if none supplied
+  if (is.null(xk)) { # space knots evenly through data
+       n<-length(x)
+       xk<-rep(0,q+m+2)
+       xk[(m+2):(q+1)]<-seq(min(x),max(x),length=q-m)
+       for (i in 1:(m+1)) {xk[i]<-xk[m+2]-(m+2-i)*(xk[m+3]-xk[m+2])}
+       for (i in (q+2):(q+m+2)) {xk[i]<-xk[q+1]+(i-q-1)*(xk[m+3]-xk[m+2])}
+     }
+  ## move m knots, xk[q-m+1],...,xk[q] (the m pre-last inner knots), to the last inner knot xk[q+1],
+  ## join these m knots with the constrained xk[q+1] knot to avoid a plateau end...
+  xk[(q-m+1):q] <- xk[q+1]
+  if (length(xk)!=nk) # right number of knots?
+   stop(paste("there should be ", nk," supplied knots"))
+  ##  get model matrix...
+  X1 <- splineDesign(xk,x,ord=m+2)
+  ## get unconstrained matrix Sigma and remove the last m+1 columns and rows...
+  Sig <- matrix(1,q,q)  ## coef summation matrix
+  Sig[upper.tri(Sig)] <-0
+  ind <- (q-m):q
+ ## Sig[,ind] <- 0; Sig[ind,] <- 0
+  Sig <- Sig[-ind,-ind]
+  X <- X1[,-ind]%*%Sig # drop (m+1) end terms, model submatrix for the scop-term
+  object$X <- X # the final model matrix
+  object$P <- list()
+  object$S <- list()
+  object$Sigma <- Sig
+
+  if (!object$fixed) # create the penalty matrix
+  { P <- diff(diag(q-length(ind)),difference=1)
+    object$P[[1]] <- P
+    object$S[[1]] <- crossprod(P)
+  }
+  object$p.ident <- c(FALSE,rep(TRUE,q-length(ind)-1)) ## p.ident is an indicator of which coefficients must be positive (exponentiated)
+  object$n.zero <- ind
+  object$rank <- ncol(object$X)-1  # penalty rank
+  object$null.space.dim <- m+1  # dim. of unpenalized space
+  object$C <- matrix(0, 0, ncol(X)) # to have no other constraints 
+  object$knots <- xk;
+  object$m <- m;
+  object$df<-ncol(object$X)     # maximum DoF 
+
+  ## get model matrix for 1st and 2nd derivatives of the smooth...
+  h <- (max(x)-min(x))/(q-m-1) ## distance between two adjacent knots
+  object$Xdf1 <- splineDesign(xk,x,ord=m+1)[,2:(q-1)]/h ## ord is by one less for the 1st derivative
+  object$Xdf2 <- splineDesign(xk,x,ord=m)[,2:(q-2)]/h^2 ## ord is by two less for the 2nd derivative
+
+  class(object)<-"mifo.smooth"  # Give object a class
+  object
+}
+
+
+## Prediction matrix for the `mifo` smooth class... 
+Predict.matrix.mifo.smooth<-function(object,data)
+## prediction method function for the `mifo' smooth class
+{ m <- object$m # spline order, m+1=3 default for cubic spline
+  q <- object$bs.dim ## object$df +1 ## ?????
+  Sig <- matrix(1,q,q)  ## coef summation matrix
+  Sig[upper.tri(Sig)] <-0
+  ind <- object$n.zero 
+  Sig[,ind] <- 0; Sig[ind,] <- 0
+  ## find spline basis inner knot range...
+  ll <- object$knots[m+2];ul <- object$knots[length(object$knots)-m-1]
+  x <- data[[object$term]]
+  n <- length(x)
+  ind <- x<=ul & x>=ll ## data in range
+  if (sum(ind)==n) { ## all in range
+     X <- spline.des(object$knots,x,m+2)$design
+     X <- X%*%Sig 
+  } else { ## some extrapolation needed 
+     ## matrix mapping coefs to value and slope at end points...
+     D <- spline.des(object$knots,c(ll,ll,ul,ul),m+2,c(0,1,0,1))$design
+     X <- matrix(0,n,ncol(D)) ## full predict matrix
+     if (sum(ind)> 0)  X[ind,] <- spline.des(object$knots,x[ind],m+2)$design ## interior rows
+     ## Now add rows for linear extrapolation...
+     ind <- x < ll 
+     if (sum(ind)>0) X[ind,] <- cbind(1,x[ind]-ll)%*%D[1:2,]
+     ind <- x > ul
+     if (sum(ind)>0) X[ind,] <- cbind(1,x[ind]-ul)%*%D[3:4,]
+     X <- X%*%Sig 
+  }
+  X
+}
+
+
+
 ########################################################
 ### Adding Monotone decreasing SCOP-spline construction 
 ########################################################
